import argparse
import os
from datetime import date
from typing import Any

from notify.config import load_config, build_channels
from notify.timeline.basic import BasicTimelineIterator, FixedDay
from notify.messages.generator import build_notification
from notify.calendar.calendar import Calendar


def parse_args() -> Any:
    parser = argparse.ArgumentParser("Daily notifications")
    parser.add_argument("--config", help="Path to config file",
                        default=[], required=True, action="append")
    parser.add_argument("--live-run", help="Run the script in live mode", action="store_true")
    parser.add_argument("--day", help="Current day", default="today", type=str)
    return parser.parse_args()


def is_real_run(args: Any) -> bool:
    return args.live_run is True \
        or os.environ.get("DO_REAL_RUN", "false") == "true"


def get_date(args: Any) -> date:
    day_var = os.environ.get("OPT_OVERRIDE_DAY")
    if day_var:
        return date.fromisoformat(day_var)
    elif args.day == "today":
        return date.today()
    else:
        return date.fromisoformat(args.day)


def main() -> None:
    args = parse_args()

    config = load_config(args.config)

    channels = list(build_channels(config))
    timeline = BasicTimelineIterator(
        workday_calendar=Calendar(config["calendar"]["rules"]),
        fixed_day=FixedDay(
            day=config["calendar"]["fixed_point"]["date"],
            moderator=config["calendar"]["fixed_point"]["moderator"],
            days_left=config["calendar"]["fixed_point"]["days_left"]
        ),
        moderator_days=config["calendar"]["moderator_days"],
        moderators=config["moderators"]
    )
    notification = build_notification(timeline, config, get_date(args))
    if not notification:
        print("No notification to send")
    else:
        if is_real_run(args):
            print("Sending messages on channels...")
            for channel in channels:
                channel.send_notification(notification)
        else:
            print("--- This is a dry-run, messages are not sent ---")
            print(notification.message)


if __name__ == "__main__":
    main()
